@@ -25,15 +25,16 @@ radio_style = {'width':'100%', 'textAlign':'right', 'margin-top':'8px'}
 dbc_color = {"blue": "primary", "grey": "secondary", "green": "success", "orange": "warning", 
             "red": "danger", "light blue": "info", "white": "light", "black": "dark"}
 
-def button(buttonName:str, buttonID:str, color:str):
+def button(buttonName:str, buttonID:str, buttonColor:str, outline:Optional[bool]=False):
     botton_layout = dbc.Row(
                         children=[
                             dbc.Col(
                                 dbc.Button(
                                     buttonName, 
                                     id=buttonID, 
-                                    n_clicks=0, 
-                                    color=color, 
+                                    n_clicks=0,
+                                    outline=outline,
+                                    color=dbc_color[buttonColor], 
                                     style=button_style0
                                 ), 
                             width=12),
@@ -105,7 +106,7 @@ def card(cardHeader: str, content: list ):
 
 
 def table(columnName:list, columnType:list, tableID:str, editable:bool,
-        row_deletable:bool, row_selectable:str, tableHeight:Optional[str]="400px", 
+        row_deletable:bool, row_selectable:str, heightTable:Optional[str]="auto",
         dropdownContent:Optional[dict]={}):
     '''
     columnType: text, numeric
@@ -135,9 +136,10 @@ def table(columnName:list, columnType:list, tableID:str, editable:bool,
                     columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i], 'presentation': 'dropdown'})
                 else:
                     columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i]})
-
-    columns.append({'id':'id','name':'id', 'type': 'text'})
     
+    style_cell_conditional = [{'if': {'column_id': columns[c]["id"]}, 'textAlign': 'left'} for c in range(len(columns))]
+    style_cell_conditional.append({'if':{'column_id': 'id',},'display':'None',})
+
     table_layout =  dash_table.DataTable(
                         id=tableID,
                         data= [],
@@ -147,12 +149,11 @@ def table(columnName:list, columnType:list, tableID:str, editable:bool,
                         row_deletable=row_deletable,
                         row_selectable=row_selectable,
                         columns=columns,
-                        fixed_rows={'headers': True},
-                        style_cell_conditional=[
-                            # {'if': {'column_id': 'time'}, 'width': '50%', 'textAlign': 'center', 'textOverflow': 'ellipsis',},
-                            # {'if': {'column_id': 'lod'},'width': '50%', 'textAlign': 'center'},
-                            {'if':{'column_id': 'id',},'display':'None',},
+                        # virtualization=True,
+                        css = [
+                               {"selector":".dropdown", "rule": "position: static"}
                         ],
+                        style_cell_conditional=style_cell_conditional,
                         style_data_conditional=[
                             {'if': {'row_index': 'odd'}, 'backgroundColor': 'rgb(248, 248, 248)'},
                             {'if':{'column_id': 'id',},'display':'None',},
@@ -162,19 +163,22 @@ def table(columnName:list, columnType:list, tableID:str, editable:bool,
                         ],
                         style_header={
                             'backgroundColor': 'rgb(230, 230, 230)',
-                            'fontWeight': 'bold'
+                            'fontWeight': 'bold',
+                            'font-size': '15px',
                         },
                         style_cell={
                             'font-size': '13px',
+                            'minWidth': 95, 
+                            # 'maxWidth': 95, 
+                            # 'width': 95
                         },
                         style_table={
                             'overflowY': 'auto',
                             'overflowX': 'auto',
                             'margin-top': '0.5em',
                             'padding-bottom': '0.5em',
-                            'height': tableHeight
+                            'height': heightTable,
                         },
-                        css=[{"selector":".dropdown", "rule": "position: static"}],
                         dropdown=dropdown,
                     )
                   
@@ -258,4 +262,61 @@ def insert_image(app, imgLocation:str, height:int):
                             style={'height': f'{height}px', 'object-fit': 'scale-down', 'Align': 'center'}),
                     ]
                 )
+    return layout
+
+def app_layout(contents:list, navigationBarColor:Optional[str]="green"):
+
+    """
+    app includes two rows: 1. navigation header, 2. contents of different pages
+    """
+    children_contents = []
+    for i in contents:
+        children_contents.append(i)
+
+    layout = html.Div(children=[
+                # Menu
+                dbc.Row(children=[
+                    dbc.Navbar([
+                        dbc.Row(
+                            children=children_contents, align="center")
+                    ], color=dbc_color[navigationBarColor],),
+                ], style=left_column_style), 
+                
+                # pages
+                dbc.Row(children=[
+                    dcc.Location(id='url'), 
+                    html.Div(id='content')
+                ], style=left_column_style)
+            ], style=left_column_style)
+    return layout
+
+
+def navigationImage(imageLocation:str, imageHeight:Optional[str]="65px"):
+    layout = dbc.Col(
+                html.Img(src=imageLocation, height=imageHeight, style={"margin-left":"20px"}), 
+            width="auto")
+    return layout
+
+def navigationBrand(brandName:Optional[str]=""):
+    layout = dbc.Col(dbc.NavbarBrand(brandName, className="ml-2", style=menu_font1), width="auto")
+    return layout
+
+def navigationLink(linkName:str, hrefName:str):
+    layout = dbc.Col(dbc.NavLink(linkName,  href=hrefName, active="exact", style=menu_font2), width="auto")
+    return layout
+
+
+def navigationDropdownMedu(menuName:str, menuItemName:list, menuItemHref:list, menuColor:Optional[str]="green"):
+    menuItems = []
+    for i in range(len(menuItemName)):
+        menuItems.append(dbc.DropdownMenuItem(menuItemName[i], href=menuItemHref[i]))
+        
+    layout = dbc.Col(
+                dbc.DropdownMenu(
+                    menuItems, 
+                    label=menuName, 
+                    color=dbc_color[menuColor], 
+                    style=menu_font2
+                ),
+            width="auto")
     return layout
\ No newline at end of file
